<?php
/******************************************************************************
 *
 *  (C) 2010 AhnLab, Inc. All rights reserved.
 *  Any part of this source code can not be copied with any method without
 *  prior written permission from the author or authorized person.
 *
 ******************************************************************************/

include_once $INC_DIR . "common_util.inc.php";

/* DB파일 경로 */
$dbpath_v3netd  = $_SERVER["DOCUMENT_ROOT"] . "/../db/v3dl.db";
$dbpath_session = $_SERVER["DOCUMENT_ROOT"] . "/../db/session.db";

/* v3net-DB 객체를 생성 */
$GLOBALS["g_db"] = new sqlLiteDb();
$g_db->db_path($dbpath_v3netd);

/* 세션-DB 객체를 생성 */
$GLOBALS["g_db_session"] = new sqlLiteDb();
$g_db_session->db_path($dbpath_session);

/* sqlite3 에러 정의 */
define(SQLITE_OK,           0);   /* Successful result */
define(SQLITE_ERROR,        1);   /* SQL error or missing database */
define(SQLITE_INTERNAL,     2);   /* Internal logic error in SQLite */
define(SQLITE_PERM,         3);   /* Access permission denied */
define(SQLITE_ABORT,        4);   /* Callback routine requested an abort */
define(SQLITE_BUSY,         5);   /* The database file is locked */
define(SQLITE_LOCKED,       6);   /* A table in the database is locked */
define(SQLITE_NOMEM,        7);   /* A malloc() failed */
define(SQLITE_READONLY,     8);   /* Attempt to write a readonly database */
define(SQLITE_INTERRUPT,    9);   /* Operation terminated by sqlite3_interrupt()*/
define(SQLITE_IOERR,       10);   /* Some kind of disk I/O error occurred */
define(SQLITE_CORRUPT,     11);   /* The database disk image is malformed */
define(SQLITE_NOTFOUND,    12);   /* NOT USED. Table or record not found */
define(SQLITE_FULL,        13);   /* Insertion failed because database is full */
define(SQLITE_CANTOPEN,    14);   /* Unable to open the database file */
define(SQLITE_PROTOCOL,    15);   /* NOT USED. Database lock protocol error */
define(SQLITE_EMPTY,       16);   /* Database is empty */
define(SQLITE_SCHEMA,      17);   /* The database schema changed */
define(SQLITE_TOOBIG,      18);   /* String or BLOB exceeds size limit */
define(SQLITE_CONSTRAINT,  19);   /* Abort due to constraint violation */
define(SQLITE_MISMATCH,    20);   /* Data type mismatch */
define(SQLITE_MISUSE,      21);   /* Library used incorrectly */
define(SQLITE_NOLFS,       22);   /* Uses OS features not supported on host */
define(SQLITE_AUTH,        23);   /* Authorization denied */
define(SQLITE_FORMAT,      24);   /* Auxiliary database format error */
define(SQLITE_RANGE,       25);   /* 2nd parameter to sqlite3_bind out of range */
define(SQLITE_NOTADB,      26);   /* File opened that is not a database file */
define(SQLITE_ROW,         100);  /* sqlite3_step() has another row ready */
define(SQLITE_DONE,        101);  /* sqlite3_step() has finished executing */

/* sqlite3 재시도 관련 정의 */
define(SQLITE_RETRY_INTERVAL, 20000000);  /* 재시도 간격 20ms */
define(SQLITE_RETRY_COUNT, 100);  /* 재시도 횟수 */

/* sqlite3-php 래퍼 클래스 */
class sqlLiteDb
{
    protected $dbhandle = null; /* sqlite3-php 클래스 핸들 */
    protected $istrans = false; /* 트랜잭션 flag */
    protected $dbpath = "";     /* db파일 경로 */

    /* constructor */
    function __construct() {
        /* TODO: */
    }

    /* destructor */
    function __destruct() {
        $this->close(); 
    }

    /**
     * @brief       sqlite 로그를 php로그로 출력한다.
     * @param[in]   errCode : 에러코드
     * @param[in]   errMessage : 에러메시지
     */
    private function sqliteLog($errCode, $errMessage) {
        $strCode = "0x".substr("00000000", strlen(dechex($errCode))).dechex($errCode);
        PHPLOG(g_LOG_DEBUG, 2, "sqlite error (".$strCode.") ".$errMessage);
    }

    /**
     * @brief       DB파일 경로 설정
     * @param[in]   s : DB파일 경로
     */
    public function db_path($s) {
        $this->dbpath = $s;
    }

    /**
     * @brief       DB파일 열기
     */
    public function open() {
        /* DB파일이 생성되어 있지 않으면 생성 */
        if (!file_exists($this->dbpath)) {
            if (!touch($this->dbpath)) {
                PHPLOG(g_LOG_DEBUG, 1, "Failed to create database".$this->dbpath);
                return;
            }
        }

        /* DB파일이 존재하는 경우 open한다 */
        if (file_exists($this->dbpath)) {
            $sleepIndex = 0;
            do {                
                $this->dbhandle = new SQLite3($this->dbpath);
                if (!$this->dbhandle) {
                    /* 실패할 경우 재시도 */
                    time_nanosleep(0, 500000000); /* 0.5sec */
                    $sleepIndex++;
                }
            } while(!$this->dbhandle && ($sleepIndex < 5));

            /* 예외처리를 받아서 처리 하도록 변경 */
            $this->dbhandle->enableExceptions(true);
        } else {
            PHPLOG(g_LOG_DEBUG, 1, "Failed to open database".$this->dbpath);
        }
    }

    /**
     * @brief       안전하게 sql문에 문자열을 포함하기 위해 escape된 문자열을 반환 
    * @param[in]   value : 문자열
     * @param[out]  escape된 String
     */
    public function escapeString($value)
    {
        return $this->dbhandle->escapeString($value);
    }

    /**
     * @brief       sqlite3-php 래퍼 클래스 내부 -> 외부로 exception throw 여부 설정
     * @param[in]   option : 설정값(true/false)
     */
    public function enableExceptions($option) {
        if ($this->dbhandle) {
            $this->dbhandle->enableExceptions($option);
        }
    }

    /**
     * @brief       DB파일 닫기
     */
    public function close() {
        /* 커밋이 되지 않은 상태로 close하면 rollback하여 변경된 데이터가 손실된다 */
        $this->rollback();

        if ($this->dbhandle) {
            $this->dbhandle->close();
        }

        $this->dbhandle = null;
    }

    /**
     * @brief       마지막으로 수행된 Query결과를 가져온다
     * @return      query수행 결과 array
     */
    public function error_code() {
        if ($this->dbhandle) {
            return $this->dbhandle->lastErrorCode();
        }

        return -1;
    }

    /**
     * @brief       트랜잭션 시작 수행
     * @remark      이미 시작되어 있는 경우 하지 않는다.
     */
    public function setTrans() {
        if ($this->dbhandle && !$this->istrans) {
            $this->istrans = true;
            $this->exec("BEGIN DEFERRED;");
        }
    }

    /**
     * @brief       트랜잭션 커밋 수행
     * @remark      트랜잭션이 시작되어 있지 않는 경우 혹은 이미 사작되어 있다면 하지 않는다.
     */
    public function commit() {
        if ($this->dbhandle && $this->istrans) {
            $this->exec("COMMIT;");
            $this->istrans = false;
        }
    }

    /**
     * @brief       트랜잭션 롤백 수행
     * @remark      트랜잭션이 시작되어 있지 않는 경우 혹은 이미 사작되어 있다면 하지 않는다.
     */public function rollback() {
        if ($this->dbhandle && $this->istrans) {
            $this->exec("ROLLBACK;");
            $this->istrans = false;
        }
    }

    /**
     * @brief       query실행 후 rows를 Sqlite3Result타입으로 반환한다.
     * @param[in]   query : 실행할 query문자열
     * @return      SQlite3Result 객체
     * @remark      클래스 내부에서만 사용
     */
    public function result($query) {
        if (!$this->dbhandle) {
            return array();
        }

        $rtn = array();
        $retryCount = 0;
        $errCode = 0;
        $errMessage = "Unknown error";

        do {
            try {
                $errCode = 0;
                $rtn = $this->dbhandle->query($query);
            } catch(Exception $e){
                $errCode = $this->error_code();
                $errMessage = $e->getMessage();
            }

            if ($errCode == SQLITE_BUSY || $errCode == SQLITE_LOCKED) {
                time_nanosleep(0, SQLITE_RETRY_INTERVAL);   /* 20ms */
                $retryCount++;
                continue;
            }

            break;
        } while($retryCount < SQLITE_RETRY_COUNT); /* 총 4초간 시도 */

        if ($errCode != 0) {
            $this->sqliteLog($errCode, $errMessage);
        }

        return $rtn;
    }

    /**
     * @brief       마지막으로 수행된 Query결과의 변경된 row갯수를 가져온다
     * @param[in]   query : 실행할 query문자열
     * @return      변경된 row갯수
     */
    public function changes($query) {
        if (!$this->dbhandle) {
            return 0;
        }

        return $this->dbhandle->changes();
    }

    /**
     * @brief       query실행 후 변경된 row갯수를 반환한다
     * @param[in]   query : 실행할 query문자열
     * @return      변경된 row갯수
     */
    public function exec($query) {
        if (!$this->dbhandle) {
            return 0;
        }

        $retryCount = 0;
        $errCode = 0;
        $errMessage = "Unknown error";

        do {
            try {
                $errCode = 0;
                $this->dbhandle->exec($query);
            } catch(Exception $e){
                $errCode = $this->error_code();
                $errMessage = $e->getMessage();
            }

            if ($errCode == SQLITE_BUSY || $errCode == SQLITE_LOCKED) {
                time_nanosleep(0, SQLITE_RETRY_INTERVAL);   /* 20ms */
                $retryCount++;
                continue;
            }

            break;
        } while($retryCount < SQLITE_RETRY_COUNT); /* 총 4초간 시도 */

        if ($errCode != 0) {
            $this->sqliteLog($errCode, $errMessage);
        }

        return $this->dbhandle->changes();
    }

    /**
     * @brief       query실행 후 1개의 row를 array타입으로 반환한다.
     * @param[in]   query : 실행할 query문자열
     * @return      single row array
     */
    public function getrow($query) {
        if (!$this->dbhandle) {
            return array();
        }

        $rtn = array();
        $retryCount = 0;
        $errCode = 0;
        $errMessage = "Unknown error";

        do {
            try {
                $errCode = 0;
                $rtn = $this->dbhandle->querySingle($query, true);
            } catch(Exception $e){
                $errCode = $this->error_code();
                $errMessage = $e->getMessage();
            }

            if ($errCode == SQLITE_BUSY || $errCode == SQLITE_LOCKED) {
                time_nanosleep(0, SQLITE_RETRY_INTERVAL);   /* 20ms */
                $retryCount++;
                continue;
            }

            break;
        } while($retryCount < SQLITE_RETRY_COUNT); /* 총 4초간 시도 */

        if ($errCode != 0) {
            $this->sqliteLog($errCode, $errMessage);
        }

        return $rtn;
    }

    /**
     * @brief       SQlite3Result 타입을 [index]["name"]array타입으로 변환 후 반환한다
     * @param[in]   results : query수행 결과를 담은 SQlite3Result객체
     * @param[in]   result_type : result type
     * @return      rows array
     * @remark      클래스 내부에서만 사용되는 메소드이다
     */
    public function fetchAll($results = null, $result_type = SQLITE3_ASSOC) {
        if ($results == null) {
            return array();
        }

        $data = array();
        $sIdx = 0;

        $retryCount = 0;
        $errCode = 0;
        $errMessage = "Unknown error";
        do {
            try {
                $errCode = 0;
                $row = $results->fetchArray($result_type);
            } catch (Exception $e) {
                $errCode = $this->error_code();
                $errMessage = $e->getMessage();
            }

            if ($errCode == SQLITE_BUSY || $errCode == SQLITE_LOCKED) {
                time_nanosleep(0, SQLITE_RETRY_INTERVAL);   /* 20ms */
                $retryCount++;
                continue;
            }

            if ($row) {
                $data[$sIdx++] = $row;
            } else {
                break;
            }
        } while($retryCount < SQLITE_RETRY_COUNT); /* 총 4초간 시도 */

        return $data;
    }

    /**
     * @brief       SQlite3Result 타입을 [index]["name"]array타입으로 변환 후 반환한다
     * @param[in]   results : query수행 결과를 담은 SQlite3Result객체
     * @param[in]   result_type : result type
     * @return      rows array
     * @remark      클래스 내부에서만 사용되는 메소드이다
     */
    public function queryAll($query, $result_type = SQLITE3_ASSOC) {
        $results = $this->result($query);
        return $this->fetchAll($results, $result_type);
    }
}
?>
